MQLST: a REXX utility to filter and list MQ objects and attributes for OS/390



     MQLST CSQ1,QUEUE,MYPROJ,CURDEPTH


     searching for <MYPROJ> in list of all <QUEUE>
     in MQSeries QManager <CSQ1>
     also retrieving <CURDEPTH>
     TEST.MYPROJ1 					0
     TEST.FOO.MYPROJ.OUTPUT   		100
     search completed successfully
     2 objects written to SYSPRINT


     MQLST CSQ1,CHS,STOPPED


     searching for <STOPPED> in list of all <CHS>
     in MQSeries QManager <CSQ1>
     also retrieving <>
     CSQ1.TO.UNIX1
     CSQ1.TO.PRODNT
     TESTNT.TO.CSQ1
     search completed successfully
     3 objects written to SYSPRINT


     CSQM201I +CSQ1 CSQMDMSG  DIS QLOCAL DETAILS
     QUEUE(MQ.PROJ1)
     TYPE(QLOCAL)
     CSQMDMSG END QLOCAL DETAILS


/*  This REXX creates a list of MQSeries objects according to    */
/*   given parameters and writes it to screen.                   */
/*  The list created is a list of objects where a certain string */
/*   (given as parameter) exists in its name or the parameter    */
/*   requested (MQparm).                                         */
/* EXAMPLE OF USAGE :                                            */
/*  MQLST <Qmanager>,<MQobj>,<String>,<MQparm>                   */
/*  For example, the command :                                   */
/*   'MQLST CSQ1,QUEUE,PROJ1,CURDEPTH'                           */
/*  Will print a table of local queues and their depth with the  */
/*   string 'PROJ1' in their name (example 'MQ.PROJ1.INPUT').    */
/*  Another example of usage, is:                                */
/*   'MQLST CSQ1,CHS,RUNNING,STATUS'                             */
/*  Will print a list of all running channels...                 */
/* PARAMETERS  :                                                 */
/*  Qmanager    - MQSeries queue manager name (without prefix).  */
/*  MQobject    - MQSeries legal object name(QUEUE,CHANNEL etc). */
/*  String      - String to search in MQSeries object names or   */
/*                in parameter MQparm.                           */
/*  MQparm      - MQSeries object parameter (CURDEPTH, STATUS)  .*/
/* MODULES USED:                                                 */
/*  RXMQVC - Rexx TSO interface for MQSeries                     */
/*           supplied as Support Pack MA19.                      */
/* REXX'S USED :                                                 */
/*  NONE.                                                        */
/* IMPORTANT NOTE:                                               */
/*  The REXX gets the objects name using the string you typed as */
/*  your 'MQobject' parameter, this causes a problem im several  */
/*  cases.                                                       */
/*  for example, if you type 'MQLST CSQ1,QLOCAL,BLA', you will   */
/*  get a blank list, this is because the output of the command  */
/*  "DISPLAY QLOCAL(*)" is as follows:                           */
/*  CSQM201I +CSQ1 CSQMDMSG  DIS QLOCAL DETAILS                  */
/*  QUEUE(MQ.PROJ1)                                              */
/*  TYPE(QLOCAL)                                                 */
/*  CSQMDMSG END QLOCAL DETAILS                                  */
/*  the word "QLOCAL" does not appear in the searched line,      */
/*  and the program cannot draw the object name from it.         */
/*  the solution is simple enough, use the word "QUEUE" and not  */
/*  QLOCAL in your query.                                        */
/* RETURN VALUE:                                                 */
/*  0/8 - (RC) whether creating the list succeeded               */
/* Updates                                                       */
/* 23/10/2000 MQACID RRNO                                        */
/*            Creation                                           */
/* 02/11/2000 MQACID RRNO                                        */
/*            Final Version                                      */
/* 19/11/2000 MQACID RRNO                                        */
/*            Added mqprm parameter                              */
/* 12/04/2001 MQACID RRNO                                        */
/*            Further commenting added                           */
   /*                 Constants Definition                       */
   TRUE                   = 1;
   FALSE                  = 0;
   NULL                   = '';
   INDEX_STRING_NOT_FOUND = 0;
   OUTPUT_TO_SCREEN       = 'SYSPRINT';
   /*                 Arguments Section                          */
   qmgr_name  = NULL;
   mqobj      = NULL;
   mqobj_str  = NULL;
   mqobj_prm  = NULL;
   PARSE ARG qmgr_name','mqobj','mqobj_str','mqobj_prm
/*
   qmgr_name = 'QM1';
   mqobj     = 'QUEUE';
   mqobj_str = 'NERL';
   mqobj_prm = 'CURDEPTH'
*/
/*
   say 'FOR DEBUG PURPOSES';
   say 'qmgr_name   <'qmgr_name'>';
   say 'mqobj_str   <'mqobj_str'>';
   say 'mqobj       <'mqobj'>';
*/
   /*                Variables Initialization                    */
   /*                       MAIN                                 */
   /* Strip parameters                                           */
   qmgr_name  = STRIP(qmgr_name);
   mqobj_str  = STRIP(mqobj_str);
   mqobj      = STRIP(mqobj);
   mqobj_prm  = STRIP(mqobj_prm);
   /* Check parameters validity                                  */
   if( qmgr_name  = NULL |,
       mqobj_str  = NULL |,
       mqobj      = NULL) then do;
   say 'MQLST Parameters Invalid.'
   say 'Rexx Usage:'
   say 'MQLST <qmgr>,<mqobj>,<string>,<mqobj_prm>';
   say 'qmgr       = MQSeries queue manager name (without prefix)'
   say 'mqobj      = CHANNEL/QLOCAL/PROCESS etc.';
   say 'string     = string to be searched in MQSeries objects';
   say 'mqobj_prm  = (OPTIONAL) MQS object parameter (exp. CURDEPTH)'
   say 'NOTE: USE UPPER CASE LETTERS'
      parms_valid = FALSE;
   end;
   else,
      parms_valid = TRUE;
   if (parms_valid = TRUE) then do;
      /* Notify user of search at hand                           */
      say 'searching for <'mqobj_str'> in list of all <'mqobj'>'
      say 'in MQSeries QManager <'qmgr_name'>'
      say 'also retrieving <'mqobj_prm'>';
      /* create object list                                      */
      objlist_received =,
         cre_objlist(qmgr_name,mqobj,mqobj_str,mqobj_prm);
      /* Write results to screen                                 */
      "EXECIO * DISKW SYSPRINT (STEM new_objlist. FINIS";
   end;
   else,
      objlist_received = FALSE;
   /* Check if object list received                              */
   if (objlist_received = TRUE) then do;
      say 'search completed successfuly';
      say new_objlist.0' objects written to SYSPRINT';
      MQlst_RC = 0;
   end;
   else do;
      say 'search was unsuccessful';
      say 'no objects objects written to SYSPRINT';
      MQlst_RC = 8;
   end;
return (MQlst_RC);
/*---------------------------------------------------------------*/
/*                    get_all_objlist                            */
/* DESCRIPTION:                                                  */
/*  This procedure gets all wanted object names from queue       */
/*   manager by using a DISPLAY * MQSeries command.              */
/*  It then creates the object list.                             */
/* PARAMETERS:                                                   */
/*  qmgr_name - MQSeries queue manager name (without prefix).    */
/*  mqobj     - MQSeries object type (CHANNEL etc.)              */
/*  mqobj_prm - MQSeries object's parameter (exp. CURDEPTH)      */
/* RETURN VALUE:                                                 */
/*  0/1 whether procedure succeeded.                             */
/*---------------------------------------------------------------*/
get_all_objlist: procedure,
                 expose TRUE,
                 expose FALSE,
                 expose INDEX_STRING_NOT_FOUND,
                 expose mqerr_msg.,
                 expose objlist.;
PARSE ARG qmgr_name,mqobj,mqobj_prm;
   /*                 Constants Definition                       */
   /*                Variables Initialization                    */
   proc_success   = FALSE;
   NO_OBJ_FOUND   = '';
   NO_PRM_FOUND   = '';
   objlist.0      = 0;
  /*                       MAIN                                 */
   /* Perform mq display all mq objects command                  */
   /* Build basic command                                        */
   mq_dis_cmd = "DIS "mqobj"(*)";
   /* If mqobject parameter stated, add to command               */
   if (mqobj_prm ^= NULL) then,
      mq_dis_cmd = mq_dis_cmd' 'mqobj_prm;
   /* Perform built command and check results                    */
   mq_cmd_success = perform_mq_cmd(qmgr_name,mq_dis_cmd);
   if (mq_cmd_success = TRUE) then do;
      /* Go through all lines in command output and extract      */
      /* queue names - from them create mq obj list              */
      do line_counter = 1 to mq_cmd_output.0;
        /* Parsing string to find object name                   */
         obj_name = NO_OBJ_FOUND;
         obj_prm  = NO_PRM_FOUND;
         /* Parse string to object name and object parameter     */
         PARSE VALUE mq_cmd_output.line_counter with,
          dummy VALUE(mqobj)'('obj_name')',
          dummy VALUE(mqobj_prm)'('obj_prm')',
          dummy;
         /* Check if string parmsed correctly                    */
         if(obj_name ^= NO_OBJ_FOUND) then do;
            next_objlist_line = objlist.0 + 1;
            objlist.next_objlist_line =,
               LEFT(STRIP(obj_name),40,' ')||' '||STRIP(obj_prm);
            objlist.0 = objlist.0 + 1;
         end;
      end;
      proc_success = TRUE;
   end;
   else do;
      /* Put error message received in stem to notify user       */
      mqerr_msg.0 = mq_cmd_output.0;
      do mqerr_line = 1 to mq_cmd_output.0;
         mqerr_msg.mqerr_line = mq_cmd_output.mqerr_line;
      end;
      proc_success = FALSE;
   end;
return (proc_success);
/*---------------------------------------------------------------*/
/*                     cre_objlist                               */
/* DESCRIPTION:                                                  */
/*  This procedure creates the wanted object list.               */
/*  This is done by going through all objects in queue manager   */
/*   and finding the requested string (given as parameter).      */
/* PARAMETERS:                                                   */
/*  qmgr_name - MQSeries queue manager name (without prefix).    */
/*  mqobj     - MQSeries object (CHANNEL, QLOCAL etc.)           */
/*  mqobj_str - The string searched in all object names.         */
/*  mqobj_prm - MQSeries object's parameter (exp. USAGE)         */
/* RETURN VALUE:                                                 */
/*  0/1 procedure success.                                       */
/*---------------------------------------------------------------*/
cre_objlist:     procedure,
                 expose TRUE,
                 expose FALSE,
                 expose INDEX_STRING_NOT_FOUND,
                 expose new_objlist.;
PARSE ARG qmgr_name,mqobj,mqobj_str,mqobj_prm;
   /*                 Constants Definition                       */
   /*                Variables Initialization                    */
   proc_success = TRUE;
   /*                       MAIN                                 */
   /* Get queue list                                             */
   objlist_received = get_all_objlist(qmgr_name,mqobj,mqobj_prm);
   if (objlist_received = TRUE) then do;
      /* Go thorugh all lines in object list and find string.       */
      /* Create object list from those lines.                       */
      new_objlist.0 = 0;
      do line_counter = 0 to objlist.0;
/*
         say 'CREobjlist: search for <'mqobj_str'>';
         say 'CREobjlist: in <'objlist.line_counter'>';
*/
        cfgidx = INDEX(objlist.line_counter, mqobj_str);
         /* if string found, add it to new object list              */
         if (cfgidx ^= INDEX_STRING_NOT_FOUND) then do;
            next_cfg_line = new_objlist.0 + 1;
            new_objlist.next_cfg_line = objlist.line_counter;
            new_objlist.0 = new_objlist.0 + 1;
/*
            say 'CREobjlist: << string found';
*/
         end;
      end;
      proc_success = TRUE;
   end;
   else,
      proc_success = FALSE;
return (proc_success);
/*-------------------------------------------------------------------*/
/*                     perform_mq_cmd                                */
/*-------------------------------------------------------------------*/
/* This procedure performs and MQSeries command and returns results  */
/*  All this using the RXMQVC module supplied by MQSeries MA19 SP    */
/*                                                                   */
/* Parameters: qmgr_name - MQSeries queue manager name (NO PREFIX)   */
/*             mq_cmd    - MQSeries command                          */
/* Return code: command success (TRUE/FALSE)                         */
/* Exposed parameters:  NONE.                                        */
/*-------------------------------------------------------------------*/
perform_mq_cmd: procedure,
                expose TRUE,
                expose FALSE,
                expose RXMQWX_mq_cmd_rcc,
                expose mq_cmd_return_code,
                expose mq_cmd_reason_code,
                expose mq_conn_return_code,
                expose mq_conn_compl_code,
                expose mq_conn_reason_code,
                expose mq_cmd_output.;
PARSE ARG qmgr_name, mq_cmd;
   /*                 Constants Definition                       */
   RXMQVC_NO_TRACE     = '';
   /*                Variables Initialization                    */
   /*                       MAIN                                 */
   /* Init RXMQVC and associated variables                           */
   rcc = RXMQVC('INIT');
   RXMQVCTRACE = RXMQVC_NO_TRACE;
   mq_cmd_output.0 = 0;
   mq_cmd_successful = FALSE;
   /* Call RXMQVC to execute command and get output                  */
   mq_cmd_rcc =,
    RXMQVC('command ',qmgr_name,mq_cmd,'mq_cmd_output.');
   mq_cmd_rcc = STRIP(mq_cmd_rcc);
   /* Init exposed variables                                         */
   RXMQVC_cmd_rcc      = mq_cmd_rcc;
   mq_cmd_return_code  = mq_cmd_output.CC;
   mq_cmd_reason_code  = mq_cmd_output.AC;
   mq_conn_return_code = word(mq_cmd_rcc,1);
   mq_conn_compl_code  = word(mq_cmd_rcc,2);
   mq_conn_reason_code = word(mq_cmd_rcc,3);
   /*                                                                */
   /* Check if connection to MQSeries successful and                 */
   /* Check if MQSeries command successful                           */
   if (mq_conn_compl_code ^= '0' |,
       mq_cmd_return_code ^= '00000000') then,
      mq_cmd_successful = FALSE;
   else,
      mq_cmd_successful = TRUE;
   /* If command not successful - print messages                     */
   if (mq_cmd_successful = FALSE) then do;
      say 'MQLST | ---------------------------------------------';
      say 'MQLST | Error executing cmd <'mq_cmd'>';
      say 'MQLST | on queue manager    <'qmgr_name'>';
      say 'MQLST | MQCONN return code  <'mq_conn_return_code'>';
      say 'MQLST | MQCONN compl code   <'mq_conn_compl_code'>';
      say 'MQLST | MQCONN reason code  <'mq_conn_reason_code'>';
      say 'MQLST | return code         <'mq_cmd_return_code'>';
      say 'MQLST | reason code         <'mq_cmd_reason_code'>';
      say 'MQLST | MQ err msg number   <'mq_cmd_output.0'>';
      say 'MQLST | ---------------------------------------------';
      do line_counter = 1 to mq_cmd_output.0;
         say 'MQLST | 'mq_cmd_output.line_counter;
      end;
      say 'MQLST | ---------------------------------------------';
      say 'MQLST | RXMQVC rcc message  <'RXMQVC_cmd_rcc'>';
      say 'MQLST | ---------------------------------------------';
   end;
   /* Terminate RXMQVC                                               */
   rcc = RXMQVC('TERM');
return (mq_cmd_successful);
